function [c_area, perimeter, cg] = apcg(x,y)

%% Called by the main code segment "shape" 
%% "apcg" computes cyclogram area, perimeter and center of
%% gravity.

%%% Copyright: Ambarish Goswami
%%% INRIA Rhone-Alpes, Grenoble, France, 1995-98

%% Accompanying paper:
%% A. Goswami
%% A new gait parameterization technique by means of cyclogram moments:
%% Application to human slope walking
%% Gait & Posture Vol. 8, No. 1 August, 1998.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% 	   AREA, PERIMETER, and C.G. CALCULATION    %%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
size_vect1 = size(x);
size_vect2 = size(y);
if size_vect1(2) ~= 1 | size_vect2(2) ~= 1
	error('the input vectors are not column vectors')
end
if size(x) ~= size(y)
	error('the input vectors are unequal')
end

nvertex = size_vect1(1)-1;

for i = 1: nvertex;
	a_sum(i) =  abs((x(i)*y(i+1) - x(i+1)*y(i)));	%are they CCW?
	if x(i+1) == x(i) 
		m_00(i) = max(y(i+1),y(i))-min(y(i+1),y(i));
		m_10(i) = x(i)*(max(y(i+1),y(i))-min(y(i+1),y(i)));
		m_01(i) = (max(y(i+1),y(i))^2-min(y(i+1),y(i))^2)/2;
	else	
		sl(i) = (y(i+1)-y(i))/(x(i+1)-x(i));
		t(i) = sqrt(1 + sl(i)^2);
		u(i) = y(i) - sl(i)*x(i);
		m_00(i) = t(i)*(max(x(i+1),x(i))-min(x(i+1),x(i)));		
		m_10(i) = t(i)*0.5*(max(x(i+1),x(i))^2-min(x(i+1),x(i))^2);
		m_01(i) = sl(i)*m_10(i) + u(i)*m_00(i);

	end	
end
c_area = 0.5*sum(a_sum);  % area is positive for a string of CCW points
perimeter = sum(m_00);
cg = [sum(m_10)/perimeter; sum(m_01)/perimeter];

%%% Based on area calculations
%j = 1;
%for i = nvertex: -1: 1;
%	mu_x(j) = (x(i) + x(i+1))*a_sum(j);
%	mu_y(j) = (y(i) + y(i+1))*a_sum(j);
%	j = j + 1;
%end
%cg = (1/(6*c_area))*[sum(mu_x); sum(mu_y)];
%cg_distance = sqrt(cg(1)^2 + cg(2)^2);
