function [m2_vector, theta_vector] = mom2(x,y)

%% Called by the main code segment "shape" 
%% "mom_2" computes the second order moments of the 
%% cyclogram -- needed for circularity, roundedness etc.

%%% Copyright: Ambarish Goswami
%%% INRIA Rhone-Alpes, Grenoble, France, 1995-98

%% Accompanying paper:
%% A. Goswami
%% A new gait parameterization technique by means of cyclogram moments:
%% Application to human slope walking
%% Gait & Posture Vol. 8, No. 1 August, 1998.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% 	 	SECOND ORDER **CENTRAL** MOMENTS    %%%%%%%%%%%
%%%%%% 	   		CALCULATION		    %%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global r2d d2r
size_vect1 = size(x);
size_vect2 = size(y);
%% 	THE INPUT VECTORS X AND Y ARE **CG-RELATED** COORDINATE 
%%	POSITIONS ON THE CONTOUR
if size_vect1(2) ~= 1 | size_vect2(2) ~= 1
	error('the input vectors are not column vectors')
end
if size(x) ~= size(y)
	error('the input vectors are unequal')
end
nvertex = size_vect1(1)-1;

%%%% Calculation of the cyclogram second moments - moments of inertia 

for i = 1: nvertex;
	if x(i+1) == x(i) 
%		i
%		disp('vertical line segment')	
		m_00(i) = max(y(i+1),y(i))-min(y(i+1),y(i));
		m_10(i) = x(i)*(max(y(i+1),y(i))-min(y(i+1),y(i)));
		m_01(i) = (max(y(i+1),y(i))^2-min(y(i+1),y(i))^2)/2;
		m_20(i) = x(i)^2*(max(y(i+1),y(i))-min(y(i+1),y(i)));
		m_11(i) = x(i)*(max(y(i+1),y(i))^2-min(y(i+1),y(i))^2)/2;
		m_02(i) = (max(y(i+1),y(i))^3-min(y(i+1),y(i))^3)/3;
	else	
		sl(i) = (y(i+1)-y(i))/(x(i+1)-x(i));
		t(i) = sqrt(1 + sl(i)^2);
		u(i) = y(i) - sl(i)*x(i);
		m_00(i) = t(i)*(max(x(i+1),x(i))-min(x(i+1),x(i)));		
		m_10(i) = t(i)*(max(x(i+1),x(i))^2-min(x(i+1),x(i))^2)/2;
		m_01(i) = sl(i)*m_10(i) + u(i)*m_00(i);
		m_20(i) = t(i)*(max(x(i+1),x(i))^3-min(x(i+1),x(i))^3)/3;
		m_11(i) = sl(i)*m_20(i) + u(i)*m_10(i);
		m_02(i) = sl(i)*m_11(i) + u(i)*m_01(i);
	end	
end

mu_00 = sum(m_00);
mu_10 = sum(m_10);
mu_01 = sum(m_01);

mu_20 = sum(m_20);
mu_11 = sum(m_11);
mu_02 = sum(m_02);


%moments = [mu_00; mu_10; mu_01; mu_20; mu_11; mu_02];


%another way to calculate the min and max second central moments
%m2_min = 0.5*((mu_20 + mu_02) - sqrt((mu_20 - mu_02)^2 + 4*mu_11^2));  
%m2_max = 0.5*((mu_20 + mu_02) + sqrt((mu_20 - mu_02)^2 + 4*mu_11^2));
%m2_vector = [m2_min; m2_max];

% The max/min m2 central moments are the eigenvalues of the 
% following matrix and the directions of the max/min moments are
% the eigenvectors of this matrix
inert = [mu_20 mu_11; mu_11 mu_02];
[p,q] = eig(inert);

%% CAUTION: To well-understand the directions of minimum 
%% and maximum moments -- The direction OF minimum moment is
%% 90 deg OFF the direction ABOUT WHICH the moment is the 
%% minimum. 

%% THUS, THE DIRECTION ``ABOUT WHICH'' THE MOMENT IS MINIMUM
%% IS "THE" DIRECTION OF MAXIMUM MOMENT

%% Below we find the direction about which the moment is minimum



if q(1,1) > q(2,2)
	theta_min = atan2(p(2,1), p(1,1))*r2d;
	m2_vector = [q(2,2);q(1,1)];
else 
	theta_min = atan2(p(2,2), p(1,2))*r2d;
	m2_vector = [q(1,1);q(2,2)];
end
theta_vector = [theta_min; theta_min+90];

%CCW angle (in degree) betn min 2nd central moment axis and the horizontal
%theta_min = atan2(+2*mu_11, +(mu_20 - mu_02))*r2d*0.5;
%CCW angle (in degree) betn max 2nd central moment axis and the horizontal
%theta_max = atan2(-2*mu_11, -(mu_20 - mu_02))*r2d*0.5;


