clear

%%% Copyright: Ambarish Goswami
%%% INRIA Rhone-Alpes, Grenoble, France, 1995-98

%% Accompanying paper:
%% A. Goswami
%% A new gait parameterization technique by means of cyclogram moments:
%% Application to human slope walking
%% Gait & Posture Vol. 8, No. 1 August, 1998.


% to read the various geometric features of the cyclograms from
% the directory value_mom, where they are stored in separate
% files, and to make plots of the individual parameters as
% functions of the ground slope
%
slopes1 = ['13'; '12'; '11'; '10'; '09'; '08'; '07'; '06'; '05';...
	'04'; '03'; '02'; '01'; '00'];
slopes2 = ['00'; '01'; '02'; '03'; '04'; '05'; '06'; '07'; '08';...
	'09'; '10'; '11'; '12'; '13'];
directions = ['d'; 'u'];
fname2 = '_mom.hkB';
fname3 = 'fx';
large_matrix = zeros(28,20);
dd = 0;
for i = 1 : 2
	fname1 = ['value_mom/fx',directions(i),'_'];

	if i == 1  
		start = 1;
	else
		start = 2;
	end
	for j = start : 14
	   dd = dd + 1;
	   if i == 1
		eval(['load ', [fname1,slopes1(j,:),fname2]]);
		large_matrix_sub =...
		 	eval(['fx',directions(i),'_',slopes1(j,:),'_mom']);
	   else
		eval(['load ', [fname1,slopes2(j,:),fname2]]);
		large_matrix_sub =...
		 	eval(['fx',directions(i),'_',slopes2(j,:),'_mom']);
	   end %if i
	   if dd == 1
		large_matrix = large_matrix_sub';
	   else
		large_matrix = [large_matrix; large_matrix_sub'];
	   end %if dd
	end %for j
end %for i

% to get rid of redundant 0 degree data
%large_matrix = [large_matrix(1:14,:); large_matrix(16:28,:)];

	

c_area 		= large_matrix(:,1);
perimeter 	= large_matrix(:,2);
cg_posn 	= large_matrix(:,3:4);
cg_distance 	= large_matrix(:,5);
m2_min_max 	= large_matrix(:,6:7);
theta_min_max 	= [large_matrix(:,8) large_matrix(:,9)];

th = theta_min_max(:,1);


vect1 = -13 : 1 : 0;
vect2 = 1 : 1 : 13;
sl = [vect1'; vect2'];

sl3 = ['-13'; '   '; '   '; '   '; '   '; ' -8'; '   '; '   '; '   '; ' -4';...
 '   '; '   '; '   '; '  0'; '   '; '   '; '   '; ' +4'; '   '; '   ';...
 '   '; ' +8'; '   '; '   '; '   '; '   '; '+13'];

	% circularity criterion
    
    circ = (4*pi*c_area)./(perimeter.^2);	

%%% 					PLOT CYCLOGRAM CIRCULARITY
n 	= 	2;
pol 	= 	polyfit(sl, circ, n);
f	= 	polyval(pol, sl);
plot(sl, f, sl, circ,'+');
axis([-15 15 0.4 1.3]);

pl_ax1 = [-15 pi/4; 15 pi/4];
pl_ax2 = [-15 pi/(3*sqrt(3)); 15 pi/(3*sqrt(3))];
pl_ax3 = [-15 1; 15 1];

lege = ['   circle   ';'eq. Triangle'; '   square   '];
leg_posx= [ -15; -14; -15];
leg_posy= [ 1.03; 0.63; 0.83];

plot(sl, f, sl, circ, '+', pl_ax1(:,1),pl_ax1(:,2),'--', pl_ax2(:,1),pl_ax2(:,2),'--', 
pl_ax3(:,1),pl_ax3(:,2),'--') 
text(leg_posx, leg_posy, lege)
xlabel('Ground slope (degree)');
ylabel('Cyclogram "circularity"');
axis([-15 15 0.0 1.1]);
axis square
grid

print 'epsfigs_tmp/circular' -deps

c_area		= c_area/((perimeter(14)^2)/(4*pi));


%%% NORMALIZED PERIMETER 

perimeter = perimeter/perimeter(14);

%%% 						PLOT NORMALIZED PERIMETER 
n 	= 	1;
pol 	= 	polyfit(sl, perimeter, n);
f	= 	polyval(pol, sl);
plot(sl, f, sl, perimeter,'+');
axis([-15 15 0.4 1.3]);
xlabel('Ground slope (degree)');
ylabel('Normalized cyclogram perimeter');
axis square
grid
print 'epsfigs_tmp/perimeter' -deps


%%% 						PLOT NORMALIZED AREA
n 	= 	2;
pol 	= 	polyfit(sl, c_area, n);
f	= 	polyval(pol, sl);
plot(sl, f, sl, c_area,'+');
axis([-15 15 0.05 0.7]);
xlabel('Ground slope (degree)');
ylabel('Normalized cyclogram area');
axis square
axis([-15 15 0.05 0.7]);
grid
print 'epsfigs_tmp/c_area' -deps


cg_distance = cg_distance/cg_distance(14);
%%% 						PLOT CYCLOGRAM CM DISTANCE

n 	= 	2;
pol 	= 	polyfit(sl, cg_distance, n);
f	= 	polyval(pol, sl);
plot(sl, f, sl, cg_distance,'+');
axis([-15 15 0.2 1.80]);
xlabel('Ground slope (degree)');
ylabel('Cyclogram CM distance from origin (degree)');
axis square
grid
print 'epsfigs_tmp/CM_distance' -deps


%%% 					PLOT OF LOCUS OF CYCLOGRAM CM 

n1 	= 	3;
pol1 	= 	polyfit(sl, cg_posn(:,1), n1);
f1	= 	polyval(pol1, sl);

n2 	= 	3;
pol2 	= 	polyfit(sl, cg_posn(:,2), n2);
f2	= 	polyval(pol2, sl);
f11 = f1 + [1 0 0 0 0 1 0 0 0 -1 0 0 0 0 0 0 0 1 0 0 0 0.5 0 0 0 0 1]';
f22 = f2 + [0 0 0 0 0 1 0 0 0  2 0 0 0 2 0 0 0 2 0 0 0 0.5 0 0 0 0 1]';

pl_ax1 = [-30 0; 60 0]; 
pl_ax2 = [0 -90; 0 15];

plot(f1, f2, '*',pl_ax1(:,1),pl_ax1(:,2),'--',pl_ax2(:,1),pl_ax2(:,2),'--')
text(f11, f22, sl3)
xlabel('Hip angle (degree)');
ylabel('Knee angle (degree)');
title('Locus of cyclogram CM with ground slope')
axis([-10 40 -60 20]);
axis square
grid

print 'epsfigs_tmp/CM_locus' -deps

%%% 						PLOT ORIENTATION ANGLE

% different parts of a curve with different polynomials
q = 17;
n1 	= 	8;
pol1 	= 	polyfit(sl(1:q), th(1:q,1), n1);
f1 	= 	polyval(pol1, sl(1:q));

n2 	= 	8;
pol2 	= 	polyfit(sl(q+2:27), th(q+2:27,1), n2);
f2	= 	polyval(pol2, sl(q+2:27));
plot(sl(1:q), f1, sl, th(:,1),'+',sl(q+2:27), f2);
axis([-15 15 80 150]);
xlabel('Ground slope (degree)');
ylabel('Angle of orientation (degree)');
axis square
grid

print 'epsfigs_tmp/theta_min' -deps


m2_mm = m2_min_max(:,2)./m2_min_max(:,1);


%%% 				PLOT RATIO OF MAX/MIN SECOND MOMENTS
n 	= 	2;
pol 	= 	polyfit(sl, m2_mm, n);
f	= 	polyval(pol, sl);
plot(sl, f, sl, m2_mm,'+');
axis([-15 15 0 20]);
xlabel('Ground slope (degree)');
ylabel('Ratio of max and min second central moments');
axis square
grid
print 'epsfigs_tmp/m2_mm' -deps


%%% 			3D PLOT OF THREE PARAMETERS
x = perimeter;
y = m2_mm;
z = cg_distance;

n = 1;
pol1 = polyfit(sl, x, n);
f1 = polyval(pol1, sl);

n = 3;
pol2 = polyfit(sl, y, n);
f2 = polyval(pol2, sl);

n = 3;
pol3 = polyfit(sl, z, n);
f3 = polyval(pol3, sl);

plot3(f1, f2, f3, f1, f2, f3,'+',f1, 15*ones(size(f2)), f3,'--',1.2*ones(size(f1)), f2, 
f3,'--');

xlabel('Normalized perimeter');
ylabel('ratio of max and min 2nd moments');
zlabel('Cyclogram CM distance from origin');


text(f1, f2, f3, sl3);
axis square
grid
print 'epsfigs_tmp/quantif' -deps



%%% 			3D PLOT OF THREE PARAMETERS (2ND PLOT)
x = c_area;
y = circ;
z =    [239
   226
   209
   218
   240
   245
   259
   238
   251
   259
   264
   272
   270
   276
   316
   294
   286
   310
   296
   272
   362
   370
   363
   314
   312
   351
   336];

n = 2;
pol1 = polyfit(sl, x, n);
f1 = polyval(pol1, sl);

n = 2;
pol2 = polyfit(sl, y, n);
f2 = polyval(pol2, sl);

n = 1;
pol3 = polyfit(sl, z/z(14), n);
f3 = polyval(pol3, sl);

plot3(f1, f2, f3, f1, f2, f3,'+',f1, 0.6*ones(size(f2)), f3,'--',0.6*ones(size(f1)), f2, 
f3,'--');


xlabel('Cyclogram area');
ylabel('Cyclogram circularity');
zlabel('Cycle duration');


text(f1, f2, f3, sl3);
axis square
grid
print 'epsfigs_tmp/quantif1' -deps


%central-oriented moments are the next
m00 		= large_matrix(:,10);
m10 		= large_matrix(:,11);	% cg_x position ~= 0
m01 		= large_matrix(:,12);	% cg_y position ~= 0
m20 		= large_matrix(:,13);	% max second moment
m11 		= large_matrix(:,14);	% m11, should be ~= 0
m02 		= large_matrix(:,15);	% min second moment
m30 		= large_matrix(:,16);
m21 		= large_matrix(:,17);
m12 		= large_matrix(:,18);
m03 		= large_matrix(:,19);
m40 		= large_matrix(:,20);



ecc = ((m20 - m02).^2)./((m20 + m02).^2); 	% eccentricity

%%% 					PLOT CYCLOGRAM ECCENTRICITY
n 	= 	2;
pol 	= 	polyfit(sl, ecc, n);
f	= 	polyval(pol, sl);
plot(sl, f, sl, ecc,'+');
xlabel('Ground slope (degree)');
ylabel('Cyclogram eccentricity');
axis([-15 15 0 1]);
axis square
grid
print 'epsfigs_tmp/eccent' -deps


th1 = m03./m30;

%%% 						RATIO OF M30/M03.

% different parts of a curve with different polynomials
q = 17;
n1 	= 	0;
pol1 	= 	polyfit(sl(1:q), th1(1:q,1), n1);
f1 	= 	polyval(pol1, sl(1:q));

n2 	= 	0;
pol2 	= 	polyfit(sl(q+2:27), th1(q+2:27,1), n2);
f2	= 	polyval(pol2, sl(q+2:27));
plot(sl(1:q), f1, sl, th1(:,1),'+',sl(q+2:27), f2);
%axis([-15 15 80 150]);
xlabel('Ground slope (degree)');
ylabel('Ratio of M03 and M30 central moments');
axis square
grid

print 'epsfigs_tmp/m03m30' -deps


th2 = m12./m21;
%%% 						RATIO OF M21/M12.

% different parts of a curve with different polynomials
q = 18;
n1 	= 	0;
pol1 	= 	polyfit(sl(1:q), th2(1:q,1), n1);
f1 	= 	polyval(pol1, sl(1:q));

n2 	= 	0;
pol2 	= 	polyfit(sl(q+3:27), th2(q+3:27,1), n2);
f2	= 	polyval(pol2, sl(q+3:27));
plot(sl(1:q), f1, sl, th2,'+', sl(q+3:27), f2);
axis([-15 15 -5 5]);
xlabel('Ground slope (degree)');
ylabel('Ratio of M12 and M21 central moments');
axis square
grid

print 'epsfigs_tmp/m12m21' -deps

%%% 					MOMENT INVARIANTS

inv1 = m20 + m02;
inv2 = (m20 - m02).^2 + 4*m11.^2;
inv3 = (m30 - 3*m12).^2 + (m03 - 3*m21).^2;
inv4 = (m30 + m12).^2 + (m03 + m21).^2;


break

con = (4/pi)^(0.25);
a = con* ( (m2_min_max(:,2).^3) ./ (m2_min_max(:,1)) ).^(0.125) ;

b = con* ( (m2_min_max(:,1).^3) ./ (m2_min_max(:,2)) ).^(0.125) ;

ratio = a./b;

