function [moments] = momn(x,y)

%% Called by the main code segment "shape" 
%% "momn" computes the higher order moments of the 
%% cyclogram.

%%% Copyright: Ambarish Goswami
%%% INRIA Rhone-Alpes, Grenoble, France, 1995-98

%% Accompanying paper:
%% A. Goswami
%% A new gait parameterization technique by means of cyclogram moments:
%% Application to human slope walking
%% Gait & Posture Vol. 8, No. 1 August, 1998.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% 	   HIGHER(>3) ORDER **CENTRAL PRINCIPAL**   %%%%%%%%%%%%%
%%%%%% 	   	MOMENT	CALCULATION		    %%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global r2d d2r
size_vect1 = size(x);
size_vect2 = size(y);
%% 	THE INPUT VECTORS X AND Y ARE **CG-RELATED** COORDINATE 
%%	POSITIONS 
%% 	ON THE CONTOUR
if size_vect1(2) ~= 1 | size_vect2(2) ~= 1
	error('the input vectors are not column vectors')
end
if size(x) ~= size(y)
	error('the input vectors are unequal')
end
nvertex = size_vect1(1)-1;

%%%% Calculation of the second moments - moments of inertia 

for i = 1: nvertex;
	if x(i+1) == x(i) 
%		i
%		disp('vertical line segment')	
		m_00(i) = max(y(i+1),y(i))-min(y(i+1),y(i));
		m_10(i) = x(i)*(max(y(i+1),y(i))-min(y(i+1),y(i)));
		m_01(i) = (max(y(i+1),y(i))^2-min(y(i+1),y(i))^2)/2;
		m_20(i) = x(i)^2*(max(y(i+1),y(i))-min(y(i+1),y(i)));
		m_11(i) = x(i)*(max(y(i+1),y(i))^2-min(y(i+1),y(i))^2)/2;
		m_02(i) = (max(y(i+1),y(i))^3-min(y(i+1),y(i))^3)/3;
		m_30(i) = x(i)^3*(max(y(i+1),y(i))^3-min(y(i+1),y(i))^3);
		m_21(i) = x(i)^2*(max(y(i+1),y(i))^2-min(y(i+1),y(i))^2)/2;
		m_12(i) = x(i)*(max(y(i+1),y(i))^3-min(y(i+1),y(i))^3)/3;
		m_03(i) = (max(y(i+1),y(i))^4-min(y(i+1),y(i))^4)/4;
		m_40(i) = x(i)^4*(max(y(i+1),y(i))^3-min(y(i+1),y(i))^3);
	else	
		sl(i) = (y(i+1)-y(i))/(x(i+1)-x(i));
		t(i) = sqrt(1 + sl(i)^2);
		u(i) = y(i) - sl(i)*x(i);
		m_00(i) = t(i)*(max(x(i+1),x(i))-min(x(i+1),x(i)));		
		m_10(i) = t(i)*(max(x(i+1),x(i))^2-min(x(i+1),x(i))^2)/2;
		m_01(i) = sl(i)*m_10(i) + u(i)*m_00(i);
		m_20(i) = t(i)*(max(x(i+1),x(i))^3-min(x(i+1),x(i))^3)/3;
		m_11(i) = sl(i)*m_20(i) + u(i)*m_10(i);
		m_02(i) = sl(i)*m_11(i) + u(i)*m_01(i);
		m_30(i) = t(i)*(max(x(i+1),x(i))^4-min(x(i+1),x(i))^4)/4;
		m_21(i) = sl(i)*m_30(i) + u(i)*m_20(i);
		m_12(i) = sl(i)*m_21(i) + u(i)*m_11(i);
		m_03(i) = sl(i)*m_12(i) + u(i)*m_02(i);
		m_40(i) = t(i)*(max(x(i+1),x(i))^5-min(x(i+1),x(i))^5)/5;
	end	
end

mu_00 = sum(m_00);
mu_10 = sum(m_10);
mu_01 = sum(m_01);

mu_20 = sum(m_20);
mu_11 = sum(m_11);
mu_02 = sum(m_02);

mu_30 = sum(m_30);
mu_21 = sum(m_21);
mu_12 = sum(m_12);
mu_03 = sum(m_03);
mu_40 = sum(m_40);
moments = [mu_00; mu_10; mu_01; mu_20; mu_11; mu_02; mu_30; mu_21; ...
		mu_12; mu_03; mu_40];
