clear
clf

%% Main code segment that make function calls to
%% "apcg", "mom_2" and "momn" to compute cyclogram
%% features and a set of geometric moments.

%%% Copyright: Ambarish Goswami
%%% INRIA Rhone-Alpes, Grenoble, France, 1995

global r2d d2r  
d2r = pi/180; % Degree to Radian conversion factor
r2d = 180/pi; % Radian to Degree conversion factor


%% Reading data of hip and knee joint angles on slope from several files

slopes1 = ['13'; '12'; '11'; '10'; '09'; '08'; '07'; '06'; '05';...
	'04'; '03'; '02'; '01'; '00'];
slopes2 = ['00'; '01'; '02'; '03'; '04'; '05'; '06'; '07'; '08';...
	'09'; '10'; '11'; '12'; '13'];
directions = ['d'; 'u'];
fname2 = '.hkB';
dd = 0;
for i = 1 : 2
	fname1 = ['cyclo_data/fx',directions(i),'_'];
	if i == 1  
		start = 1;
	else
		start = 2;
	end
	for j = start : 14
%	for j = 7 : 7
	   dd = dd + 1;
	   if i == 1
		eval(['load ', [fname1,slopes1(j,:),fname2]]);
		cur = eval(['fx',directions(i),'_',slopes1(j,:)]);
		x = cur(:,1);
	  	y = cur(:,2);
		clear cur
		eval(['clear ', ['fx',directions(i,:),'_',slopes1(j,:)]]);
	   else
		eval(['load ', [fname1,slopes2(j,:),fname2]]);
		cur = eval(['fx',directions(i),'_',slopes2(j,:)]);
		x = cur(:,1);
	  	y = cur(:,2);
		eval(['clear ', ['fx',directions(i,:),'_',slopes2(j,:)]]);
		clear cur
	  end % if i

 % Extracting Max and Min joint flexions

	fea(dd,1) = max(y);			%min_knee extension
	fea(dd,2) = min(y);			%min_knee flexion
	fea(dd,3) = max(x);			%max_hip flexion
	fea(dd,4) = min(x);			%max_hip extension
	fea(dd,5) = abs(fea(dd,2) - fea(dd,1)); %knee_range
	fea(dd,6) = abs(fea(dd,4) - fea(dd,3)); %hip_range
	fea(dd,7) = max(size(x));		%cycle duration


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%   function call to apcg.m    %%%%%%%%%%%%%%%%%%%%
%% to calculate cyclogram area, perimeter and cg_position %%%%

[c_area, perimeter, cg_posn] = apcg(x,y);

cg_posn;
% Distance of Cyclogram cg from origin
cg_distance = sqrt(cg_posn(1)^2 + cg_posn(2)^2);  


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% 	   	CHANGE OF COORDINATE ORIGIN         %%%%%%%%%%%%%
%%%%%%	  The coordinate origin is shifted to the CG   %%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
x_bar = x - cg_posn(1)*ones(size(x));
y_bar = y - cg_posn(2)*ones(size(y));


%%%%%%%%%%%%%%%    function call to mom2.m    %%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%    to calculate second order moments %%%%%%%%%%%%%%%


[m2_min_max, theta_min_max] = mom2(x_bar,y_bar);
%theta_min_max(1)=rem((theta_min_max(1)+180*ones(size(theta_min_max(1)))), 180);
for kk = 1 : max(size(theta_min_max))
	if theta_min_max(kk,1) < -90
		theta_min_max(kk,1) = theta_min_max(kk,1) + 180;
	end
end

ang = theta_min_max(1)*d2r;
rot_matrix = [cos(ang) sin(ang); -sin(ang) cos(ang)];
rotated = rot_matrix* [x_bar y_bar]';


%%%%%% The coordinate frames are rotated to max-min moments %%%%%%
x_rot = rotated(1,:)';
y_rot = rotated(2,:)';
clear rotated;


%%%%%%%%%%%%%%%   function call to momn.m    %%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%	  to calculate higher order moments  %%%%%%%%%%%%%%%

[n_moments] = momn(x_rot,y_rot);

gmat = [c_area; perimeter; cg_posn; cg_distance; m2_min_max; theta_min_max; n_moments];

	end % j
end % i


%%% 						PLOT Cycle duration

vect1 = -13 : 1 : 0;
vect2 = 1 : 1 : 13;
sl = [vect1'; vect2'];

n 	= 	1;
pol 	= 	polyfit(sl, fea(:,5), n);
f	= 	polyval(pol, sl);
plot(sl, f, sl, fea(:,5),'+');
axis([-15 15 20 120]);
xlabel('Ground slope (degree)');
ylabel('Range of knee movement (degree)');
axis square
grid
print 'epsfigs_tmp/knee_range' -deps

break

%%%% to calculate the max/min hip knee angles, range etc. 



	fea(dd,1) = max(y); 			%min_knee extension
	fea(dd,2) = min(y);			%min_knee flexion
	fea(dd,3) = max(x);			%max_hip flexion
	fea(dd,4) = min(x);			%max_hip extension
	fea(dd,5) = abs(fea(dd,2) - fea(dd,1)); %knee_range
	fea(dd,6) = abs(fea(dd,4) - fea(dd,3)); %hip_range
	fea(dd,7) = max(size(x));		%cycle duration


% different parts of a curve fit with different polynomials
%n1 	= 	2;
%pol1 	= 	polyfit(sl(1:14), fea(1:14,2), n1);
%f1 	= 	polyval(pol1, sl(1:14));

%n2 	= 	2;
%pol2 	= 	polyfit(sl(15:27), fea(15:27,2), n2);
%f2	= 	polyval(pol2, sl(15:27));
%plot(sl(1:14), f1, '--', sl, fea(:,2),sl(15:27), f2, '--');


% Augment the data vector to close the contour
	x = [x; x(1,:)];
	y = [y; y(1,:)];




% fea(:,7) = fea(:,7)/fea(14,7);	% normalization of cycle duration
n 	= 	1;
pol 	= 	polyfit(sl, fea(:,5), n);
f	= 	polyval(pol, sl);

plot(sl, f, '--', sl, fea(:,5));

break

%%%%%%%%% To plot the range of the joints %%%%%%%%%%%%%%%%%%

vect1 = -13 : 1 : 0;
vect2 = 1 : 1 : 13;
sl = [vect1'; vect2'];

xlabel('Ground slope (degree)');
ylabel('Range of knee movement (degree)');
axis('square');





	jjj = 0;
	for jj = -110 : 0.5 : 30
		jjj = jjj + 1;
		if jj > fea(dd,2) & jj < fea(dd,1)
			gen1(dd, jjj) = 1;
		else
			gen1(dd, jjj) = 0;
		end % if j

	end %for jj	

	jjj = 0;
	for jj = -30 : 0.5 : 75
		jjj = jjj + 1;
		if jj > fea(dd,4) & jj < fea(dd,3)
			gen2(dd, jjj) = 1;
			color(dd, jjj) = 'c';
		else
			gen2(dd, jjj) = 0;
			color(dd, jjj) = 'g';
		end % if j

	end %for jj


vect1 = -13 : 1 : 0;
vect2 = 1 : 1 : 13;
sl = [vect1'; vect2'];
jj = [-110 : 0.5 : 30];
%mesh(jj, sl, gen1, color)
axis('square')
pcolor(jj, sl, gen1)
%grid





vect1 = -13 : 1 : 0;
vect2 = 1 : 1 : 13;
sl = [vect1'; vect2'];

n 	= 	3;
pol 	= 	polyfit(sl, fea(:,1), n);
f	= 	polyval(pol, sl)

plot(sl, f, '--', sl, fea(:,1));


axis('square')
grid

break




%plot(sl, area);
grid
title(' ');
xlabel('Ground slope (degree)');
ylabel('Area (square degree)');
%ylabel('Cycle duration (frame numbers)');
%print 'epsfigs/c_area' -deps


break

%pl_ax1 = [-30 0; 60 0]; 
%pl_ax2 = [0 -90; 0 10];
%disp_ax1 = [-30 0; 60 0]; 
%disp_ax2 = [0 -90; 0 10];
subplot(2,2,1)
%plot(x,y,pl_ax1(:,1),pl_ax1(:,2),'--',pl_ax2(:,1),pl_ax2(:,2),'--');

plot(x, y);

%axis([-30 60 -95 15]);
axis('equal','square');
title('03 degree down')
xlabel('Hip Angle (degrees)');
ylabel('Knee Angle (degrees)');


%pl_ax1 = [-25 0; 25 0]; 
%pl_ax2 = [0 -55; 0 35];
subplot(2,2,2);
%plot(x_bar,y_bar,pl_ax1(:,1),pl_ax1(:,2),'--',pl_ax2(:,1),pl_ax2(:,2),'--');
%axis([-30 30 -50 50]);
plot(x_bar,y_bar);
axis('equal','square');
title('CG displaced')
xlabel('Hip Angle (degrees)');
ylabel('Knee Angle (degrees)');


break


pl_ax1 = [-25 0; 25 0]; 
pl_ax2 = [0 -45; 0 30];
plot(x_rot,y_rot,pl_ax1(:,1),pl_ax1(:,2),'--',pl_ax2(:,1),pl_ax2(:,2),'--');
axis([-50 50 -50 50])
axis('equal','square');
title('Axes rotated')
xlabel('Hip Angle (degrees)');
ylabel('Knee Angle (degrees)');


